@@ -1,5 +1,7 @@
 #!/usr/bin.env python
 
+from __future__ import absolute_import
+from __future__ import unicode_literals
 import os
 import os.path
 import argparse
